@@ -1,31 +1,37 @@
 #!/usr/bin/env python2
 # vim:fileencoding=utf-8
-from __future__ import (unicode_literals, division, absolute_import,
-                        print_function)
+# License: GPLv3 Copyright: 2013, Kovid Goyal <kovid at kovidgoyal.net>
+from __future__ import absolute_import, division, print_function, unicode_literals
 
-__license__ = 'GPL v3'
-__copyright__ = '2013, Kovid Goyal <kovid at kovidgoyal.net>'
-
-import os, posixpath
+import os
+import posixpath
 from binascii import hexlify
-from collections import OrderedDict, defaultdict, Counter
+from collections import Counter, OrderedDict, defaultdict
 from functools import partial
 
 import sip
 from PyQt5.Qt import (
-    QWidget, QTreeWidget, QGridLayout, QSize, Qt, QTreeWidgetItem, QIcon, QFont,
-    QStyledItemDelegate, QStyle, QPixmap, QPainter, pyqtSignal, QMenu, QTimer,
-    QDialogButtonBox, QDialog, QLabel, QLineEdit, QVBoxLayout, QScrollArea, QInputDialog,
-    QRadioButton, QFormLayout, QSpinBox, QListWidget, QListWidgetItem, QCheckBox)
+    QCheckBox, QDialog, QDialogButtonBox, QFont, QFormLayout, QGridLayout, QIcon,
+    QInputDialog, QLabel, QLineEdit, QListWidget, QListWidgetItem, QMenu, QPainter,
+    QPixmap, QRadioButton, QScrollArea, QSize, QSpinBox, QStyle, QStyledItemDelegate,
+    Qt, QTimer, QTreeWidget, QTreeWidgetItem, QVBoxLayout, QWidget, pyqtSignal
+)
 
-from calibre import human_readable, sanitize_file_name_unicode, plugins
-from calibre.ebooks.oeb.base import OEB_STYLES, OEB_DOCS
-from calibre.ebooks.oeb.polish.container import guess_type, OEB_FONTS
-from calibre.ebooks.oeb.polish.replace import get_recommended_folders
+from calibre import human_readable, plugins, sanitize_file_name_unicode
+from calibre.ebooks.oeb.base import OEB_DOCS, OEB_STYLES
+from calibre.ebooks.oeb.polish.container import OEB_FONTS, guess_type
 from calibre.ebooks.oeb.polish.cover import (
-    get_cover_page_name, get_raster_cover_name, is_raster_image)
-from calibre.gui2 import error_dialog, choose_files, question_dialog, elided_text, choose_save_file
-from calibre.gui2.tweak_book import current_container, tprefs, editors
+    get_cover_page_name, get_raster_cover_name, is_raster_image
+)
+from calibre.ebooks.oeb.polish.css import add_stylesheet_links
+from calibre.ebooks.oeb.polish.replace import get_recommended_folders
+from calibre.gui2 import (
+    choose_dir, choose_files, choose_save_file, elided_text, error_dialog,
+    question_dialog
+)
+from calibre.gui2.tweak_book import (
+    CONTAINER_DND_MIMETYPE, current_container, editors, tprefs
+)
 from calibre.gui2.tweak_book.editor import syntax_from_mime
 from calibre.gui2.tweak_book.templates import template_for
 from calibre.utils.icu import sort_key
@@ -69,7 +75,9 @@ def get_bulk_rename_settings(parent, number, msg=None, sanitize=sanitize_file_na
     d.l = l = QFormLayout(d)
     d.setLayout(l)
     d.prefix = p = QLineEdit(d)
-    prefix = prefix or {k:v for k, __, v in CATEGORIES}.get(category, _('Chapter-'))
+    default_prefix = {k:v for k, __, v in CATEGORIES}.get(category, _('Chapter-'))
+    previous = tprefs.get('file-list-bulk-rename-prefix', {})
+    prefix = prefix or previous.get(category, default_prefix)
     p.setText(prefix)
     p.selectAll()
     d.la = la = QLabel(msg or _(
@@ -85,6 +93,8 @@ def get_bulk_rename_settings(parent, number, msg=None, sanitize=sanitize_file_na
 
     if d.exec_() == d.Accepted:
         prefix = sanitize(unicode(d.prefix.text()))
+        previous[category] = prefix
+        tprefs.set('file-list-bulk-rename-prefix', previous)
         num = d.num.value()
         fmt = '%d'
         if leading_zeros:
@@ -210,6 +220,17 @@ class FileList(QTreeWidget):
             }.iteritems()}
         self.itemActivated.connect(self.item_double_clicked)
 
+    def mimeTypes(self):
+        ans = QTreeWidget.mimeTypes(self)
+        ans.append(CONTAINER_DND_MIMETYPE)
+        return ans
+
+    def mimeData(self, indices):
+        ans = QTreeWidget.mimeData(self, indices)
+        names = (idx.data(0, NAME_ROLE) for idx in indices if idx.data(0, MIME_ROLE))
+        ans.setData(CONTAINER_DND_MIMETYPE, '\n'.join(filter(None, names)).encode('utf-8'))
+        return ans
+
     @property
     def current_name(self):
         ci = self.currentItem()
@@ -466,6 +487,9 @@ class FileList(QTreeWidget):
             m.addAction(QIcon(I('save.png')), _('Export %s') % n, partial(self.export, cn))
             if cn not in container.names_that_must_not_be_changed and cn not in container.names_that_must_not_be_removed and mt not in OEB_FONTS:
                 m.addAction(_('Replace %s with file...') % n, partial(self.replace, cn))
+            if num > 1:
+                m.addAction(QIcon(I('save.png')), _('Export all %d selected files') % num, self.export_selected)
+
             m.addSeparator()
 
             m.addAction(QIcon(I('modified.png')), _('&Rename %s') % n, self.edit_current_item)
@@ -642,9 +666,7 @@ class FileList(QTreeWidget):
     def selectedIndexes(self):
         ans = QTreeWidget.selectedIndexes(self)
         if self.ordered_selected_indexes:
-            # The reverse is needed because Qt's implementation of dropEvent
-            # reverses the selectedIndexes when dropping.
-            ans = list(sorted(ans, key=lambda idx:idx.row(), reverse=True))
+            ans = list(sorted(ans, key=lambda idx:idx.row()))
         return ans
 
     def dropEvent(self, event):
@@ -718,6 +740,14 @@ class FileList(QTreeWidget):
         if path:
             self.export_requested.emit(name, path)
 
+    def export_selected(self):
+        names = self.selected_names
+        if not names:
+            return
+        path = choose_dir(self, 'tweak_book_export_selected', _('Choose location'))
+        if path:
+            self.export_requested.emit(names, path)
+
     def replace(self, name):
         c = current_container()
         mt = c.mime_map[name]
@@ -725,7 +755,7 @@ class FileList(QTreeWidget):
         filters = [oext]
         fname = _('Files')
         if mt in OEB_DOCS:
-            fname = _('HTML Files')
+            fname = _('HTML files')
             filters = 'html htm xhtm xhtml shtml'.split()
         elif is_raster_image(mt):
             fname = _('Images')
@@ -798,6 +828,9 @@ class NewFileDialog(QDialog):  # {{{
         self.name = n = QLineEdit(self)
         n.textChanged.connect(self.update_ok)
         l.addWidget(n)
+        self.link_css = lc = QCheckBox(_('Automatically add style-sheet links into new HTML files'))
+        lc.setChecked(tprefs['auto_link_stylesheets'])
+        l.addWidget(lc)
         self.err_label = la = QLabel('')
         la.setWordWrap(True)
         l.addWidget(la)
@@ -827,6 +860,7 @@ class NewFileDialog(QDialog):  # {{{
             self.do_import_file(path[0])
 
     def do_import_file(self, path, hide_button=False):
+        self.link_css.setVisible(False)
         with open(path, 'rb') as f:
             self.file_data = f.read()
         name = os.path.basename(path)
@@ -849,6 +883,7 @@ class NewFileDialog(QDialog):  # {{{
         if not self.name_is_ok:
             return error_dialog(self, _('No name specified'), _(
                 'You must specify a name for the new file, with an extension, for example, chapter1.html'), show=True)
+        tprefs['auto_link_stylesheets'] = self.link_css.isChecked()
         name = unicode(self.name.text())
         name, ext = name.rpartition('.')[0::2]
         name = (name + '.' + ext.lower()).replace('\\', '/')
@@ -856,6 +891,10 @@ class NewFileDialog(QDialog):  # {{{
         if not self.file_data:
             if mt in OEB_DOCS:
                 self.file_data = template_for('html').encode('utf-8')
+                if tprefs['auto_link_stylesheets']:
+                    data = add_stylesheet_links(current_container(), name, self.file_data)
+                    if data is not None:
+                        self.file_data = data
                 self.using_template = True
             elif mt in OEB_STYLES:
                 self.file_data = template_for('css').encode('utf-8')
